<?php

namespace Drupal\file_sync;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\FileTransfer\FileTransferException;
use Drupal\Core\FileTransfer\FTPExtension;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Simple FTP client service using the core FTPExtension.
 */
class FTPClient implements FTPClientInterface {

  use StringTranslationTrait;

  /**
   * The FTP extension.
   *
   * @var \Drupal\Core\FileTransfer\FTPExtension
   */
  protected $ftp;

  /**
   * Current FTP connection resource.
   *
   * @var resource
   */
  protected $connection;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The stream wrapper manager.
   *
   * @var Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * Construct.
   *
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager
   *   The stream wrapper manager.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   */
  public function __construct(
    StreamWrapperManagerInterface $stream_wrapper_manager,
    FileSystemInterface $file_system) {
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public function connect(array $settings): self {
    $this->ftp = FTPExtension::factory($settings['jail'], $settings);
    $this->ftp->connect();
    $this->connection = $this->ftp->connection;

    $passive_mode = ($settings['advanced']['passive']) ?? FALSE;
    if ($passive_mode) {
      if(!$this->setPassiveMode()) {
        throw new \Exception('Failed to enable FTP passive mode.');
      }
    }

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getConnection() {
    if ($this->ftp === NULL) {
      throw new \Exception('No FTP connection has been initialized. Use FTPClient::connect().');
    }
    return $this->connection;
  }

  /**
   * {@inheritdoc}
   */
  public function list(string $directory = '.'): array {
    $list = ftp_nlist($this->getConnection(), $directory);
    if (!$list) {
      throw new \Exception("Failed to list the desired FTP directory:" . $directory);
    }
    return $list;
  }

  /**
   * {@inheritdoc}
   */
  public function getFile(string $source_file, string $destination_file, $return = FALSE): ?string {
    $connection = $this->getConnection();

    if (!@ftp_get($connection, $destination_file, $source_file, FTP_BINARY)) {
      throw new FileTransferException($this->t('Cannot download @source to @destination', [
        '@source' => $source_file,
        '@destination' => $destination_file,
      ]));
    }

    if ($return) {
      if (!touch($destination_file) || !is_readable($destination_file)) {
        throw new \Exception($this->t('Failed to download the "%source" to path "%destination"', [
          '%source' => $source_file,
          '%destination' => $destination_file,
        ]));
      }

      // Open the downloaded temp file and read its contents.
      $file_contents = '';
      $file_resource = fopen($destination_file, 'r');
      if (!$file_resource) {
        throw new \Exception($this->t('Failed to open file "%destination" for reading.', [
          '%destination' => $destination_file,
        ]));
      }

      while (!feof($file_resource)) {
        $file_contents .= fgets($file_resource);
      }
      fclose($file_resource);

      return $file_contents;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getDirectory(string $source, string $destination): void {
    $connection = $this->getConnection();

    // If stream wrapper is used convert scheme to realpath.
    if ($this->streamWrapperManager->getScheme($destination)) {
      $destination = $this->fileSystem->realpath($destination);
    }

    if ($source !== '.') {
      if (ftp_chdir($connection, $source) === FALSE) {
        return;
      }
      if (!(is_dir($destination)) && !mkdir($destination)) {
        throw new \Exception('Fail to create the destination directory: ' . $destination);
      }
    }

    $files = ftp_nlist($connection, $source);
    foreach ($files as $file) {
      if ($file == '.' || $file == '..') {
        continue;
      }

      $source_file = $source . DIRECTORY_SEPARATOR . $file;
      $destination_file = $destination . DIRECTORY_SEPARATOR . $file;

      // FTP extension has no efficient way to test if current file
      // is a directory. Suppress the warning message if file is hit.
      if (@ftp_chdir($connection, $file)) {
        ftp_chdir($connection, '..');
        $this->getDirectory($source_file, $destination_file);
      } else {
        ftp_get($connection, $destination_file, $source_file, FTP_BINARY);
      }
    }

    ftp_chdir($connection, '..');
    chdir('..');
  }

  /**
   * Enable FTP passive mode.
   */
  protected function setPassiveMode(): bool {
    return ftp_pasv($this->getConnection(), TRUE);
  }

}
