<?php

namespace Drupal\file_sync\Plugin\file_sync;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file_sync\FTPClientInterface;
use Drupal\file_sync\Plugin\OperationPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\File\FileSystemInterface;

/**
 * Defines plugin for downloading directory from FTP server.
 *
 * @OperationPlugin(
 *   id = "file_sync_ftp",
 *   label = "FTP - Directory sync"
 * )
 */
class FTPDirectorySync extends OperationPluginBase {

  use StringTranslationTrait;

  /**
   * The FTP client.
   *
   * @var \Drupal\file_sync\FTPClientInterface
   */
  protected $ftpClient;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a \Drupal\Component\Plugin\PluginBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    LoggerChannelFactoryInterface $logger,
    FTPClientInterface $ftp,
    FileSystemInterface $file_system) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger);
    $this->ftpClient = $ftp;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('file_sync.ftp_client'),
      $container->get('file_system'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function execute(array $settings) {
    if ($this->getStatus()) {
      try {
        $ftp = $this->ftpClient->connect($settings + [
          'advanced' => [
            'port' => $settings['port'],
            'hostname' => $settings['hostname'],
            'passive' => $settings['passive'],
          ],
        ]);

        if ($settings['log']) {
          $this->logger->info($this->t('Downloading FTP %source to %destination.', [
            '%source' => $settings['source_path'],
            '%destination' => $settings['destination_path'],
          ]));
        }

        $ftp->getDirectory($settings['source_path'], $settings['destination_path']);

        if ($settings['log']) {
          $this->logger->info($this->t('Finished downloading FTP %source to %destination.', [
            '%source' => $settings['source_path'],
            '%destination' => $settings['destination_path'],
          ]));
        }
      }
      catch (\Exception $exception) {
        $this->logger->error($exception->getMessage());
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $settings = $this->getEntity()->getPluginSettings();
    $form['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#required' => TRUE,
      '#default_value' => $settings['username'] ?? '',
    ];
    $form['password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#required' => TRUE,
      '#default_value' => $settings['password'] ?? '',
    ];
    $form['hostname'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Hostname'),
      '#required' => TRUE,
      '#default_value' => $settings['hostname'] ?? '',
    ];
    $form['port'] = [
      '#type' => 'number',
      '#title' => $this->t('Port'),
      '#default_value' => $settings['port'] ?? 21,
    ];
    $form['jail'] = [
      '#type' => 'textfield',
      '#title' => $this->t('FTP jail'),
      '#description' => $this->t('Full path to path where operations are restricted to.'),
      '#required' => TRUE,
      '#default_value' => $settings['jail'] ?? '',
    ];
    $form['source_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Source path'),
      '#description' => $this->t('Full path to directory on the FTP server.'),
      '#required' => TRUE,
      '#default_value' => $settings['source_path'] ?? '',
    ];
    $form['destination_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Destination path'),
      '#description' => $this->t('Full path to directory on the local server.'),
      '#required' => TRUE,
      '#default_value' => $settings['destination_path'] ?? '',
    ];
    $form['passive'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Passive'),
      '#description' => $this->t('Use passive FTP connection.'),
      '#default_value' => $settings['passive'] ?? 1,
    ];
    $form['log'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Log activity'),
      '#description' => $this->t('Create watchdog messages about FTP plugin activity.'),
      '#default_value' => $settings['log'] ?? 0,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Check if we have permissions to write into "destination_path".
    $destination_path = $form_state->getValue('destination_path');
    if ($destination_path) {
      if (!$this->fileSystem->prepareDirectory($destination_path, FileSystemInterface::CREATE_DIRECTORY)) {
        $form_state->setErrorByName('destination_path', $this->t(
          '<strong>Destination path</strong> could not be created.'
        ));
      }
    }
  }

}
